import pygrib, sys

if len(sys.argv) < 3:
    sys.stdout.write("""
cnvgrib2to1 <grib2 filename> <grib1 filename>\n\n""")
    raise SystemExit
   
grbs = pygrib.open(sys.argv[1])
f = open(sys.argv[2],'wb')

sys.stdout.write('converting %s from grib2 to grib1 (%s) ...\n' % (sys.argv[1],sys.argv[2]))

nmsg = 0
for grb in grbs:
    try:
        grb.editionNumber=1 
        nmsg = nmsg + 1
    except:
        sys.stdout.write('cannot convert message %s\n' % grb.messagenumber)
        continue
    f.write(grb.tostring())

sys.stdout.write('%s messages out of %s converted\n' % (nmsg,grbs.messages))
grbs.close()
f.close()



import pygrib, sys
pygrib.tolerate_badgrib_on()
if len(sys.argv) < 2:
    sys.stdout.write("'grib_list <grib_filename>' to get long listing\n")
    sys.stdout.write("'grib_list <grib_filename> -s' to get short listing\n")
    sys.stdout.write("'grib_list <grib_filename> -m' to get short listing plus data min/max\n")
    sys.exit(1)
fname = sys.argv[1]
short = False
medium = False
if len(sys.argv) > 2:
    if sys.argv[2] == '-s': short=True
    if sys.argv[2] == '-m': medium=True
grbs = pygrib.open(fname)
if short:
    for grb in grbs:
        sys.stdout.write(repr(grb)+'\n')
elif medium:
    for grb in grbs:
        try:
            grb.expand_grid(False); data = grb.values
            sys.stdout.write(repr(grb)+':min/max=%g/%g'%(data.min(), data.max())+'\n')
        except:
            sys.stdout.write(repr(grb)+':min/max=NO DATA'+'\n')
else:
    for grb in grbs:
        sys.stdout.write('------message %d------\n' % grb.messagenumber)
        for k in grb.keys():
            if k.startswith('mars'): continue
            if k in ['values','codedValues','packedValues','unpackedValues']: continue
            if grb.is_missing(k) and k not in ['analDate','validDate']:
                sys.stdout.write('%s = MISSING\n' % k)
            else:
                try:
                    v = getattr(grb,k)
                    sys.stdout.write('%s = %s\n'%(k,v))
                except:
                    sys.stdout.write('%s = NOT FOUND\n'%k)
sys.stdout.write('packing = %s\n' % grb.packingType)
grbs.close()



import pygrib, sys

if len(sys.argv) < 3:
    sys.stdout.write("""
cnvgrib1to2 <grib1 filename> <grib2 filename> <packing scheme>
<packing_scheme> is optional - can be 'grid_simple', 'grid_complex',
'grid_complex_spatial_differencing', 'grid_jpeg', or 'grid_png'. 
Default is 'grid_jpeg'\n\n""")
    raise SystemExit
   
grbs = pygrib.open(sys.argv[1])
f = open(sys.argv[2],'wb')

if len(sys.argv) > 3:
   grb2packing = sys.argv[3]
else:
   grb2packing = 'grid_jpeg' # default is jpeg2000 

sys.stdout.write('converting %s from grib1 to grib2 (%s) with %s packing ...\n' % (sys.argv[1],sys.argv[2],grb2packing))

nmsg = 0
for grb in grbs:
    try:
        grb.editionNumber=2 
        grb.packingType = grb2packing
        nmsg = nmsg + 1
    except:
        sys.stdout.write('cannot convert message %s\n' % grb.messagenumber)
        continue
    f.write(grb.tostring())

sys.stdout.write('%s messges out of %s converted\n' % (nmsg,grbs.messages))
grbs.close()
f.close()



"""
 command line program to change packing scheme in a grib2 file

 'grib_repack -h' for help.
"""
from optparse import OptionParser
import sys
import pygrib

usage = """
 %prog [options] grib2filein grib2fileout

 repack the data in grib2filein with a different packing scheme

 example:

'%prog -p3 grib2filein grib2fileout' converts grib2filein to complex packing with spatial differencing, saves result to grib2fileout."""

parser = OptionParser(usage=usage)
parser.add_option("-p", default=False, action="store",dest="ipack",
                        help="desired packing scheme - 0 for simple packing, 1 for simple matrix packing, 2 for complex packing, 3 for complex packing with spatial differencing, 4 for no packing (ieee), 40 for JPEG2000 packing, 41 for PNG packing, 61 for simple log pre-processing")

(options, args) = parser.parse_args()

if not options.ipack:
    sys.stdout.write('must specify a packing scheme with -p\n')
    sys.exit(1)

ipack = int(options.ipack)
if ipack not in [0,1,2,3,4,40,41,61]:
    raise ValueError('packing scheme must be one of 0,1,2,3,4,40,41,61')

# open grib file
try:
    filein = args[0]
    fileout = args[1]
except:
    sys.stdout.write('need to specify input and output grib2 filenames\n')
    sys.exit(1)
sys.stdout.write('input grib file: %s\n' % filein)
grbs = pygrib.open(filein)

# open the file for output.
sys.stdout.write('output grib file: %s\n' % fileout)
f=open(fileout,'wb')

#  "grid_simple"                       = { dataRepresentationTemplateNumber = 0;     }
#  "spectral_complex"                  = { dataRepresentationTemplateNumber = 51; spectralType=1; spectralMode=1; }
#  "spectral_simple"                   = { dataRepresentationTemplateNumber = 50; spectralType=1; spectralMode=1; }
#  "grid_simple_matrix"                = { dataRepresentationTemplateNumber = 1;     }
#  "grid_complex"                      = { dataRepresentationTemplateNumber = 2;     }
#  "grid_complex_spatial_differencing" = { dataRepresentationTemplateNumber = 3;     }
#  "grid_jpeg"                         = { dataRepresentationTemplateNumber = 40000; }
#  "grid_jpeg"                         = { dataRepresentationTemplateNumber = 40;    }
#  "grid_png"                          = { dataRepresentationTemplateNumber = 40010; }
#  "grid_png"                          = { dataRepresentationTemplateNumber = 41;    }
#  "grid_ieee"                         = { dataRepresentationTemplateNumber = 4;     }
#  "grid_simple_log_preprocessing"     = { dataRepresentationTemplateNumber = 61;    }
#  "grid_second_order"                 = { dataRepresentationTemplateNumber = 0;     }
#  "grid_second_order_different_width" = { dataRepresentationTemplateNumber = 0;  }

for grb in grbs:
    if grb['editionNumber'] != 2:
        sys.stdout.write('not a GRIB2 message, skipping ..\n')
        continue
    if grb['dataRepresentationTemplateNumber'] == ipack:
        sys.stdout.write('no repacking required, skipping ..\n')
    grb['dataRepresentationTemplateNumber'] = ipack
    msg = grb.tostring()
    f.write(msg)

f.close()
grbs.close()
